//----------------------------------------------------------------------------------
//	FILE:			converter_PWM_Setup.c
//	Description:	PWM setup for full-bridge DC-DC converter
//	Version: 		1.0
//  Target:  		TMS320F2806x family
//	Type: 			Device Dependent
//  Copyright       ELK21 course 2011
//  Date:  			June 2012 
//----------------------------------------------------------------------------------

#include "F2806x_Device.h"          // DSP2860x Headerfile Include File
#include "F2806x_EPWM_defines.h" 	// Defines specifics to EPWM

#include "DSP28x_Project.h" 							// Device Headerfile and Examples Include File

#define TBCTLVAL  0x200E           // up-down count, timebase=SYSCLKOUT

void InitEPWMs(void)
{
 
 //---------------------------------------------------------------------
 //--- Must disable the clock to the ePWM modules if you       
 //--- want all ePMW modules synchronized.
 //---------------------------------------------------------------------
	asm(" EALLOW");						// Enable EALLOW protected register access
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;
	asm(" EDIS");						// Disable EALLOW protected register access

//---------------------------------------------------------------------
//--- Configure ePWM1 for 50 kHz symmetric PWM on EPWM1A and EPWM1B pin
//    Also configure ePWM1 to trigger the ADC    
//---------------------------------------------------------------------
		
	// Setup counter mode
	
	EPwm1Regs.TBCTL.bit.CTRMODE = TB_FREEZE;		 	// Stop-freeze counter operation until all bits are set
	EPwm1Regs.TBCTL.bit.FREE_SOFT=0x3;					//1x - Free run
	EPwm1Regs.TBCTL.bit.PHSEN = TB_DISABLE;				// Disable phase loading - for MASTER mode
	EPwm1Regs.TBCTL.bit.PRDLD = TB_SHADOW;				//  reload PRD on counter=0
	EPwm1Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_DISABLE;		// sync-out disabled
	EPwm1Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;          	
	EPwm1Regs.TBCTL.bit.CLKDIV = TB_DIV1;              

	EPwm1Regs.TBPRD = 800;								// Set timer period for up-and-down-count	
	EPwm1Regs.TBCTR = 0x0000;							// Clear counter
	
	EPwm1Regs.TBPHS.half.TBPHS = 0x0000;				// Set Phase register to zero
	EPwm1Regs.CMPA.half.CMPA=EPwm1Regs.TBPRD/2;			// Initial duty ratio = 50 %
	
	// Setup shadowing
	EPwm1Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;			// shadow mode
	EPwm1Regs.CMPCTL.bit.LOADAMODE=CC_CTR_ZERO;			// Load on Zero
	// Set actions- up-count
	EPwm1Regs.AQCTLA.bit.CAD = AQ_CLEAR;				// PWM1A low (AQ_CLEAR) when CMPA=TBCNT on down count (CAD)
	EPwm1Regs.AQCTLA.bit.CAU = AQ_SET;         			// PWM1A high (AQ_SET) when CMPA =TBCNT on up count (CAU)
   
	EPwm1Regs.AQCTLB.bit.CAD = AQ_SET;       			// PWM1B high (AQ_SET) when CMPA = TBCNT on down count (CAD)
	EPwm1Regs.AQCTLB.bit.CAU = AQ_CLEAR;    			// PWM1B low (AQ_CLEAR) when CMPA = TBCNT on up count (CAU)
   
	// set deadband, Trip-zone and PWM_chopper units
	EPwm1Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;		// enable Dead-band module
	EPwm1Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC;			// Active Hi complementary
	EPwm1Regs.DBFED = 200;								// FED = 200 TBCLKs = 2.4us
	EPwm1Regs.DBRED = 200;								// RED = 200 TBCLKs
	EPwm1Regs.PCCTL.bit.CHPEN = 0;						// PWM chopper unit disabled
	EPwm1Regs.TZDCSEL.all = 0x0000;						// All trip zone and DC compare actions disabled	   
                    
    // Configure Event-Trigger Prescale Register (ETPS)
	// Refer table 3-64 TRM page 358
	EPwm1Regs.ETPS.all = 0x0000;						// Configure SOCA
	EPwm1Regs.ETPS.bit.SOCAPRD=ET_1ST;  				// 01 = generate SOCA on first event
	
	// bit 15-14     00:     EPWMxSOCB, read-only
	// bit 13-12     00:     SOCBPRD, don't care
	// bit 11-10     00:     EPWMxSOCA, read-only
	// bit 9-8       01:     SOCAPRD, 01 = generate SOCA on first event
	// bit 7-4       0000:   reserved
	// bit 3-2       00:     INTCNT, don't care
	// bit 1-0       00:     INTPRD, don't care
	
	
	// Configure Event-Trigger Selection Register (ETSEL)
	//// Refer table 3-63 TRM page 357
	
 	EPwm1Regs.ETSEL.all = 0x0000;
 	EPwm1Regs.ETSEL.bit.SOCAEN=1;						// Start of Conversion A Enable
 	EPwm1Regs.ETSEL.bit.SOCASEL= ET_CTR_ZERO;			// 010=SOCA when TBCTR = ZERO

	// bit 15        0:      SOCBEN, 0 = disable SOCB
	// bit 14-12     000:    SOCBSEL, don't care
	// bit 11        1:      SOCAEN, 1 = enable SOCA
	// bit 10-8      010:    SOCASEL, 010 = SOCA on PRD event
	// bit 7-4       0000:   reserved
	// bit 3         0:      INTEN, 0 = disable interrupt
	// bit 2-0       000:    INTSEL, don't care
                    
                    
                    
         
    EPwm1Regs.TBCTL.bit.CTRMODE=TB_COUNT_UPDOWN;		//Enable the time-base in count up down mode	
    
//---------------------------------------------------------------------
//--- Configure ePWM2 for 50 kHz symmetric PWM on EPWM2A and EPWM2B pin    
//---------------------------------------------------------------------
		
	// Setup counter mode
	EPwm2Regs.TBCTL.bit.CTRMODE = TB_FREEZE;			// Stop-freeze counter operation
	EPwm2Regs.TBCTL.bit.FREE_SOFT=0x3;					//1x - Free run
	EPwm2Regs.TBCTL.bit.PHSEN = TB_DISABLE;				// Disable phase loading - for MASTER mode
	EPwm2Regs.TBCTL.bit.PRDLD = TB_SHADOW;				//  reload PRD on counter=0
	EPwm2Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_DISABLE;		// sync-out disabled
	EPwm2Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;          
	EPwm2Regs.TBCTL.bit.CLKDIV = TB_DIV1;              

	EPwm2Regs.TBPRD = 800;								// Set timer period for up-count	
	EPwm2Regs.TBCTR = 0x0000;							// Clear counter
	
	EPwm2Regs.TBPHS.half.TBPHS = 0x0000;				// Set Phase register to zero
	
	EPwm2Regs.CMPA.half.CMPA=EPwm2Regs.TBPRD/2;			//Duty ratio = 50 %
	
	// Setup shadowing
	EPwm2Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;			// shadow mode
	EPwm2Regs.CMPCTL.bit.LOADAMODE=CC_CTR_ZERO;			// Load on Zero
	
	// Set actions- up-count
	EPwm2Regs.AQCTLA.bit.CAD = AQ_SET;					// PWM2A high when CMPA = TBCNT on down count (CAD)
	EPwm2Regs.AQCTLA.bit.CAU = AQ_CLEAR;				// PWM2A low when CMPA = TBCNT on up count (CAU)
   
	EPwm2Regs.AQCTLB.bit.CAD = AQ_CLEAR;				// PWM2B low when CMPA = TBCNT on down count (CAD)
	EPwm2Regs.AQCTLB.bit.CAU = AQ_SET;					// PWM2B high when CMPA = TBCNT on up count (CAU)
   
    // set deadband, Trip-zone and PWM_chopper units
	EPwm2Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;		// enable Dead-band module
	EPwm2Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC;			// Active Hi complementary
	EPwm2Regs.DBFED = 200; 								// FED = 200 TBCLKs
	EPwm2Regs.DBRED = 200;

	EPwm2Regs.PCCTL.bit.CHPEN = 0;						// PWM chopper unit disabled
	EPwm2Regs.TZDCSEL.all = 0x0000;						// All trip zone and DC compare actions disabled	   
                           
    EPwm2Regs.TBCTL.bit.CTRMODE=TB_COUNT_UPDOWN;		//Enable the time-base in count up down mode	
    
    
//---------------------------------------------------------------------
//--- Configure ePWM3 for 50 kHz symmetric PWM on EPWM3A and EPWM3B pin    
//---------------------------------------------------------------------
		

		
	// Setup counter mode
	
	EPwm3Regs.TBCTL.bit.CTRMODE = TB_FREEZE	;			// Stop-freeze counter operation
	EPwm3Regs.TBCTL.bit.FREE_SOFT=0x3;					// 1x - Free run
	EPwm3Regs.TBCTL.bit.PHSEN = TB_DISABLE;				// Disable phase loading - for MASTER mode
	EPwm3Regs.TBCTL.bit.PRDLD = TB_SHADOW;				// reload PRD on counter=0
	EPwm3Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_DISABLE;    	// sync-out disabled
	EPwm3Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;          
	EPwm3Regs.TBCTL.bit.CLKDIV = TB_DIV1;              

	EPwm3Regs.TBPRD = 800;								// Set timer period for up-count	
	EPwm3Regs.TBCTR = 0x0000;							// Clear counter
	
	EPwm3Regs.TBPHS.half.TBPHS = 0x0000;				// Set Phase register to zero
	
	EPwm3Regs.CMPA.half.CMPA=EPwm3Regs.TBPRD;			//Duty ratio: Can this become a sinusoidal signal? Set to an initial value to 50% and change it in the interrupt.
	
	// Setup shadowing
	EPwm3Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;			// shadow mode
	EPwm3Regs.CMPCTL.bit.LOADAMODE=CC_CTR_ZERO;			// Load on Zero
	
	// Set actions- up-count
	EPwm3Regs.AQCTLA.bit.CAD = AQ_CLEAR;
	EPwm3Regs.AQCTLA.bit.CAU = AQ_SET;					// Clear PWM1A on CMPA=counter, up count
   
	EPwm3Regs.AQCTLB.bit.CAD = AQ_SET;
	EPwm3Regs.AQCTLB.bit.CAU = AQ_CLEAR; 
   

	// set deadband, Trip-zone and PWM_chopper units

	EPwm3Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;		// enable Dead-band module
	EPwm3Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC;			// Active Hi complementary
	EPwm3Regs.DBFED = 200;								// FED = 200 system clocks
	EPwm3Regs.DBRED = 200;
	EPwm3Regs.PCCTL.bit.CHPEN = 0;						// PWM chopper unit disabled
	EPwm3Regs.TZDCSEL.all = 0x0000;						// All trip zone and DC compare actions disabled	   
                    
         
    EPwm3Regs.TBCTL.bit.CTRMODE=TB_COUNT_UPDOWN;		//Enable the time-base in count up down mode	    
    
    
    

	
	//---------------------------------------------------------------------
	// To start the ePWM Time-base clock (TBCLK) within the ePWM modules, 
	// the TBCLKSYNC bit in PCLKCR0 must also be set.
	//--- Enable the clocks to the ePWM module.                   
	//--- Note: this should be done after all ePWM modules are configured
	//--- to ensure synchronization between the ePWM modules.	
	//---------------------------------------------------------------------
	
	asm(" EALLOW");										// Enable EALLOW protected register access
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;				// TBCLK to ePWM modules enabled
	asm(" EDIS");										// Disable EALLOW protected register access
	
}// end InitEPWMs()

