/***********************************
File name: DC_md_main-F2806x_1.c
Purpose: Control a DC motor drive
**********************************************/

#include "F2806x_Device.h"


void DeviceInit(void);  // defined in DC_md_DevInit_F2806x.c
void InitEPWMs(void);   // defined in DC_md_PWM_Setup.c
void InitEQEP(void);    // defined in DC_md_EQep_Setup.c
void InitADC(void);		// defined in DC_md_Adc.c
void InitEPWM4(void);	// defined in DC_md_Adc.c   (used for verification)

//define interrupt service routines
interrupt void EQEP1_ISR(void);
interrupt void ADCINT1_ISR(void);

Uint32 EndlessLoopCounter = 0;
int QepIntCounter = 0;
int AdcIntCounter = 0;

// Speed controller parameters
int speedArray[50];
int actualSpeed = 0;
int setSpeed = 0;
int speedError=0;
double kpS=0;
double kiS=0;
double pTermS=0;
double iStateS=0;
int iStateSMax=100;
int iStateSMin=-100;
double iTermS=0;
int speedController=0;



// Current controller parameters
int setCurrent=0; 
int currentError=0;
double kpI=0;
double kiI=0;
double pTermI=0;
double iStateI=0;
int iStateIMax=100;
int iStateIMin=-100;
double iTermI=0;
int currentController=0;


//ADC ISR buffer array
Uint16 AdcBuf[50][8]={0};





void main(void)
{ 

// Initialize System Control
// Enable Peripheral Clocks, GPIO
// Initialize the PIE control registers to their default state.
// The default state is all PIE interrupts disabled and flags
// are cleared.
 DeviceInit();
 InitEPWMs();	//Initialize PWM modules
 InitADC();		// Initialize ADC
 InitEQEP();	// Initialize QEP
 InitEPWM4();	// Initialize EPWM4

 //Enable the interrupt
 EALLOW;
 	PieVectTable.EQEP1_INT=&EQEP1_ISR;
 	PieVectTable.ADCINT1=&ADCINT1_ISR;
 EDIS;
 
 PieCtrlRegs.PIEIER1.bit.INTx1=1;
 PieCtrlRegs.PIEIER5.bit.INTx1=1;
 IER |= M_INT1;
 IER |=	M_INT5;
 EINT;	
 	    
  for(;;)  //infinite loop
  {
    asm(" NOP"); 
    if (EndlessLoopCounter++ >= 4294967295)
    {
       EndlessLoopCounter=0;
    }    
  }
  
}// end of main



//EQP ISR:
interrupt void EQEP1_ISR(void)
{
	static int index=0;	//Initialize an index which counts every interrupt
	
 	int border = 70; 	//indicates the QPOSLAT value at the border between the high and low speed measurements
	
	//Speed calculation
	if (EQep1Regs.QEPSTS.bit.QDF==1) 		// If forward motor rotation
	{	
		//if high speed, equation 3.19 (rpm):
		if (EQep1Regs.QPOSLAT>border) actualSpeed = 1171875*EQep1Regs.QPOSLAT/EQep1Regs.QUPRD;
		// if low speed, equation 3.21 (rpm):
		else if(EQep1Regs.QPOSLAT<=border) actualSpeed = 75000000/EQep1Regs.QCPRDLAT/1024; 
	}
	else if (EQep1Regs.QEPSTS.bit.QDF==0)	// If reverse motor direction
	{
		Uint32 POSLAT = EQep1Regs.QPOSMAX - EQep1Regs.QPOSLAT;
		//if high speed, equation 3.20 (rpm):
		if(POSLAT >border) actualSpeed = - (1171875*POSLAT/EQep1Regs.QUPRD); 
		//if low speed, equation 3.21 (rpm):
		else if (POSLAT<=border) actualSpeed = - 75000000/EQep1Regs.QCPRDLAT/1024;
	}
	//Store the results in a circular array
	speedArray[index]=actualSpeed;
	
	//Speed PI controller:
	speedError = setSpeed - actualSpeed;			//Calculate the error
	pTermS  =  kpS*speedError;						// proportional term
	iStateS += speedError;							// the sum of the speed errors
	if (iStateS>iStateSMax)iStateS=iStateSMax;		// anti-windup
	if (iStateS<iStateSMin)iStateS=iStateSMin;
	iTermS =  kiS*iStateS*EQep1Regs.QUPRD/80000000; // integral term. Sampling period=EQep1Regs.QUPRD/80MHz
	speedController=pTermS+iTermS;					// speed controller's output
	
	setCurrent+=speedController;					// update setCurrent
	if(setCurrent >4096)setCurrent = 4096;			// avoid setcurrent to become too high. 
													// This is the maximum value of the input current. 
		
	index++;
	if (index==50) index=0;

		
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP5; 	// Acknowledge this interrupt to receive more interrupts
	EQep1Regs.QCLR.bit.UTO=1;					// Clears unit time out interrupt flag
	EQep1Regs.QCLR.bit.INT=1;					// Clears the interrupt flag and enables further interrupts 
												// to be generated if an event flags is set to 1
	EQep1Regs.QEPSTS.bit.UPEVNT=1; 				// Clear unit position event flag to allow for more latches 
												// of the QCTMR value into the QCPRD register
	
	
	//QEP interrupt counter
	QepIntCounter++;
	if (QepIntCounter==200)	QepIntCounter=0;
	
	
}//end of ISR QEP


	


interrupt void ADCINT1_ISR(void)
{
	static int index = 0;
		
	//Store the converted values in a circular array:
	AdcBuf[index][0]=AdcResult.ADCRESULT0;
	AdcBuf[index][1]=AdcResult.ADCRESULT1;
	AdcBuf[index][2]=AdcResult.ADCRESULT2;
	AdcBuf[index][3]=AdcResult.ADCRESULT3;
	AdcBuf[index][4]=AdcResult.ADCRESULT4;
	AdcBuf[index][5]=AdcResult.ADCRESULT5;
	AdcBuf[index][6]=AdcResult.ADCRESULT6;
	AdcBuf[index][7]=AdcResult.ADCRESULT7;
	
	index ++;
	if (index == 50) index =0;

	static int indexB = 1;
	indexB ++;
	
	//Current/torque loop
	if (indexB==10) //To make the current loop calculate for every 10th interrupt, making the sampling time for the current loop on order of magnitude lower than the switching frequency
	{	
		//calculates the output of the current control loop
		currentError=setCurrent-AdcResult.ADCRESULT2;	//calculates the current error
		pTermI=kpI*currentError;						//proportional term
		iStateI+=currentError;							//sums up the current errors
		if(iStateI>iStateIMax) iStateI=iStateIMax;		//anti-windup
		if(iStateI<iStateIMin) iStateI=iStateIMin;
		iTermI=kiI*iStateI*EPwm1Regs.TBPRD/4000000;		//integral term: sampling period=10*EPwm1Regs.TBPRD*2/80MHz
		currentController=pTermI+iTermI;				//current controller output
		
		
		//calculate the new duty ratio	
		EPwm1Regs.CMPA.half.CMPA -= currentController;  
		EPwm2Regs.CMPA.half.CMPA=EPwm1Regs.CMPA.half.CMPA;	
		
		indexB=1;	
	}
	
		
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;		// Must acknowledge the PIE group
	AdcRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;		// Clear ADCINT1 flag to enable further interrupts
	
	//ADC interrupt counter
	  AdcIntCounter++;
      if (AdcIntCounter >=200)
        AdcIntCounter=0;
        
} //end of ADCINT1_ISR
